<?php

/**
 * @file
 * OpenID Connect client for Akamai.
 */

/**
 * Implements OpenID Connect Client plugin for Akamai(Janrain).
 */
class OpenIDConnectClientAkamai extends OpenIDConnectClientBase {

  /**
   * Using the SHA256 hashing function.
   */
  const CODE_CHALLENGE_METHOD = 'sha256';
  const CODE_CHALLENGE_METHOD_AKAMAI_NAME = 'S256';

  /**
   * The name of the session variable in which the "code verifier" is stored.
   */
  const CODE_VERIFIER_VARIABLE = 'akamai_auth_code_verifier';

  /**
   * Current language.
   *
   * @var string
   */
  private $language;

  public function __construct($name, $label, array $settings) {
    parent::__construct($name, $label, $settings);
    global $language;
    $this->language = $language;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm() {
    $form = parent::settingsForm();

    $form['domain_endpoint'] = [
      '#title' => t('Domain Name For Endpoints'),
      '#description' => t('Please provide a domain name for endpoints, example https://DomainName.com'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('domain_endpoint'),
      '#weight' => -10,
    ];

    $form['customer_id'] = [
      '#title' => t('Customer ID'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('customer_id'),
    ];

    $form['client_secret']['#disabled'] = TRUE;
    $form['client_secret']['#type'] = 'hidden';
    $form['client_secret']['#required'] = FALSE;

    $form['enable_sso'] = [
      '#title' => t('Enable SSO login'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('enable_sso'),
    ];

    $form['sso_cookie_age'] = [
      '#type' => 'textfield',
      '#title' => t('SSO login interval in seconds'),
      '#default_value' => $this->getSetting('sso_cookie_age') ?: 3600,
      '#size' => 8,
      '#description' => t('The time interval between SSO login attempts, in seconds.Must be 60 seconds or greater.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getEndpoints() {
    $uri = trim($this->getSetting('domain_endpoint'), '/ ') . '/' . $this->getSetting('customer_id');
    return array(
      'authorization' => $uri . '/login/authorize',
      'token' => $uri . '/login/token',
      'userinfo' => $uri . '/profiles/oidc/userinfo',
    );
  }

  /**
   * {@inheritdoc}
   */
  public function authorize($scope = 'openid email profile', $login_verification = NULL) {
    $redirect_uri = OPENID_CONNECT_REDIRECT_PATH_BASE . '/' . $this->name;
    $url_options = array(
      'query' => array(
        'client_id' => $this->getSetting('client_id'),
        'response_type' => 'code',
        'scope' => $scope,
        'redirect_uri' => url($redirect_uri, array('absolute' => TRUE)),
        'state' => openid_connect_create_state_token(),
        'code_challenge' => $this->getCodeChallenge(),
        'code_challenge_method' => self::CODE_CHALLENGE_METHOD_AKAMAI_NAME,
        'ui_locales' => $this->getLocale(),
      ),
    );

    if ($login_verification) {
      $url_options['query']['prompt'] = 'none';
    }

    $endpoints = $this->getEndpoints();
    // Clear $_GET['destination'] because we need to override it.
    unset($_GET['destination']);
    drupal_goto($endpoints['authorization'], $url_options);
  }

  /**
   * {@inheritdoc}
   */
  private function getLocale(): string {
    $current_language_id = $this->language->language;
    $current_language_component = explode('-', $current_language_id, 2);
    if (isset($current_language_component[1])) {
      $current_language_component[1] = strtoupper($current_language_component[1]);
      $current_language_id = implode('-', $current_language_component);
    }
    return $current_language_id;
  }

  /**
   * {@inheritdoc}
   */
  public function retrieveTokens($authorization_code) {
    // Exchange `code` for access token and ID token.
    $redirect_uri = OPENID_CONNECT_REDIRECT_PATH_BASE . '/' . $this->name;
    $post_data = array(
      'code' => $authorization_code,
      'client_id' => $this->getSetting('client_id'),
      'client_secret' => $this->getSetting('client_secret'),
      'redirect_uri' => url($redirect_uri, array('absolute' => TRUE)),
      'grant_type' => 'authorization_code',
      'code_verifier' => $this->getCodeVerifier(),
    );
    $request_options = array(
      'method' => 'POST',
      'data' => drupal_http_build_query($post_data),
      'timeout' => 15,
      'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
    );

    $endpoints = $this->getEndpoints();

    $response = drupal_http_request($endpoints['token'], $request_options);
    if (!isset($response->error) && $response->code == 200) {
      $response_data = drupal_json_decode($response->data);
      $tokens = array(
        'id_token' => $response_data['id_token'],
        'access_token' => $response_data['access_token'],
      );
      if (array_key_exists('expires_in', $response_data)) {
        $tokens['expire'] = REQUEST_TIME + $response_data['expires_in'];
      }
      if (array_key_exists('refresh_token', $response_data)) {
        $tokens['refresh_token'] = $response_data['refresh_token'];
      }
      return $tokens;
    }
    else {
      openid_connect_log_request_error(__FUNCTION__, $this->name, $response);
      return FALSE;
    }
  }

  /**
   * Create a “code verifier” and Code Challenge(PKCE).
   *
   * Code verifier a random string of 43 to 128(126 for Drupal Generator)
   * characters.
   */
  protected function getCodeChallenge() {
    $code_verifier = $this->generateCodeVerifier(mt_rand(43, 126));

    $_SESSION[self::CODE_VERIFIER_VARIABLE] = $code_verifier;

    // Then base64-url encodes the hashed string
    // to create a “code challenge” string.
    $hash = strtoupper(hash(self::CODE_CHALLENGE_METHOD, $code_verifier));
    return str_replace(['+', '/', '='], ['-', '_', ''], base64_encode(pack('H*', $hash)));
  }

  /**
   * {@inheritdoc}
   */
  protected function generateCodeVerifier($length = 8) {
    $values = array_merge(range(65, 90), range(97, 122), range(48, 57));
    $max = count($values) - 1;
    $code_verifier = chr(mt_rand(97, 122));
    for ($i = 1; $i < $length; $i++) {
      $code_verifier .= chr($values[mt_rand(0, $max)]);
    }
    return $code_verifier;
  }

  /**
   * {@inheritdoc}
   */
  protected function getCodeVerifier() {
    return $_SESSION[self::CODE_VERIFIER_VARIABLE];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsFormValidate($form, &$form_state, $error_element_base) {
    parent::settingsFormValidate($form, $form_state, $error_element_base);

    // Whether a client ID is given.
    if (empty($form_state['values']['client_id'])) {
      form_set_error($error_element_base . 'client_id', t('The client ID is missing for Akamai.'));
    }

    // Whether a customer ID is given.
    if (empty($form_state['values']['customer_id'])) {
      form_set_error($error_element_base . 'customer_id', t('The customer ID is missing for Akamai.'));
    }

    // Whether a customer ID is given.
    if (empty($form_state['values']['domain_endpoint'])) {
      form_set_error($error_element_base . 'domain_endpoint', t('The Domain name for endpoint is missing for Akamai.'));
    }

  }

}
